# Contributing Guidelines
Thank you for your interest in contributing to our project. Whether it is a bug report, new feature, correction, or 
additional documentation, we greatly value feedback and contributions from our community.

Please read through this document before submitting any issues or pull requests to ensure we have all the necessary 
information to effectively respond to your bug report or contribution.

## Reporting Bugs/Feature Requests
We welcome you to use the GitHub issue tracker to report bugs or suggest features.

When filing an issue, please check existing open issues to make sure somebody else hasn't already reported the issue. 
Please try to include as much information as you can. Details like these are incredibly useful:

* A reproducible test case or series of steps
* The version of Logstash being used
* Any modifications you've made relevant to the bug
* Anything unusual about your environment or deployment, such as additional Logstash plugins that do not come with a
standard build

When you submit log examples, please cleanse logs of any identifiable information before submitting. 

## Contributing and Testing
Contributions via pull requests are much appreciated. Before sending us a pull request, please ensure that:

1. You are working against the latest source on the *master* branch.
2. You check existing open, and recently merged, pull requests to make sure someone else hasn't addressed the problem already.
3. You open an issue to discuss any significant work - we would hate for your time to be wasted.

To send us a pull request, please:

1. Fork the repository.
2. Modify the source; please focus on the specific change you are contributing. If you also reformat all the code, it 
will be hard for us to focus on your change.
3. Commit to your fork using clear commit messages.
4. Send us a pull request, answering any default questions in the pull request interface.
5. Pay attention to any automated CI failures reported in the pull request, and stay involved in the conversation.

We will be implementing a robust test suite in the future. Please stay tuned if you open multiple PRs over time, because
the testing strategy might have changed.

GitHub provides additional documentation on [forking a repository](https://help.github.com/articles/fork-a-repo/) and
[creating a pull request](https://help.github.com/articles/creating-a-pull-request/).

## Code of Conduct
OpenSIEM-Logstash-Parsing operates under the [Cargill Code of
Conduct](https://github.com/Cargill/code-of-conduct/blob/master/code-of-conduct.md).

## Security issue notifications
If you discover a potential security issue in this project, please notify us via email at 
[Abuse Cargill](abuse@cargill.com). Please do **not** create a public github issue.

## Licensing
OpenSIEM-Logstash-Parsing software is licensed under the [Apache License Version 2.0](LICENSE)
software license.
